@@ -21,29 +21,25 @@
  *         Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  */
 
-const Lang = imports.lang;
-const Mainloop = imports.mainloop;
-const Signals = imports.signals;
+const GLib = imports.gi.GLib;
+const GObject = imports.gi.GObject;
 const Gettext = imports.gettext;
-const _ = imports.gettext.gettext;
-
-const GtkClutter = imports.gi.GtkClutter;
 const Gio = imports.gi.Gio;
 const Gtk = imports.gi.Gtk;
-const GLib = imports.gi.GLib;
+const GtkClutter = imports.gi.GtkClutter;
+const Lang = imports.lang;
+const _ = imports.gettext.gettext;
 
-const Main = imports.main;
 const Format = imports.format;
 const Geoclue = imports.geoclue;
 const GeocodeService = imports.geocodeService;
 const MainWindow = imports.mainWindow;
-const Notification = imports.notification;
 const NotificationManager = imports.notificationManager;
-const Utils = imports.utils;
 const Path = imports.path;
-const Settings = imports.settings;
 const PlaceStore = imports.placeStore;
 const RouteService = imports.routeService;
+const Settings = imports.settings;
+const Utils = imports.utils;
 
 // used globally
 let application = null;
@@ -53,10 +49,27 @@ let notificationManager = null;
 let routeService = null;
 let geoclue = null;
 let geocodeService = null;
+let networkMonitor = null;
 
 const Application = new Lang.Class({
     Name: 'Application',
     Extends: Gtk.Application,
+    Properties: {
+        'connected': GObject.ParamSpec.boolean('connected',
+                                               '',
+                                               '',
+                                               GObject.ParamFlags.READABLE |
+                                               GObject.ParamFlags.WRITABLE)
+    },
+
+    set connected(p) {
+        this._connected = p;
+        this.notify('connected');
+    },
+
+    get connected() {
+        return this._connected;
+    },
 
     _init: function() {
         Gettext.bindtextdomain('gnome-maps', Path.LOCALE_DIR);
@@ -66,6 +79,22 @@ const Application = new Lang.Class({
         GLib.set_application_name(_("Maps"));
 
         this.parent({ application_id: 'org.gnome.Maps' });
+        this._connected = false;
+    },
+
+    _checkNetwork: function() {
+        let addr = new Gio.NetworkAddress({ hostname: 'tile.openstreetmap.org',
+                                            port: 80 });
+
+        networkMonitor.can_reach_async(addr, null, (function(networkMonitor, res) {
+            try {
+                if (networkMonitor.can_reach_finish(res))
+                    this.connected = true;
+            } catch(e) {
+                this.connected = false;
+                Utils.debug('Connection failed: ' + e.message);
+            }
+        }).bind(this));
     },
 
     _onQuitActivate: function() {
@@ -101,10 +130,9 @@ const Application = new Lang.Class({
         application = this;
         this._initServices();
 
-        Utils.initActions(this, [{
-            properties: { name: 'quit' },
-            signalHandlers: { activate: this._onQuitActivate }
-        }], this);
+        Utils.addActions(this, {
+            'quit': { onActivate: this._onQuitActivate.bind(this) }
+        });
 
         this._initPlaceStore();
         this._initAppMenu();
@@ -115,6 +143,9 @@ const Application = new Lang.Class({
         routeService   = new RouteService.GraphHopper();
         geoclue        = new Geoclue.Geoclue();
         geocodeService = new GeocodeService.GeocodeService();
+        networkMonitor = Gio.NetworkMonitor.get_default();
+        networkMonitor.connect('network-changed',
+                               this._checkNetwork.bind(this));
     },
 
     _createWindow: function() {
@@ -122,7 +153,7 @@ const Application = new Lang.Class({
             return;
 
         Gtk.IconTheme.get_default().append_search_path(Path.ICONS_DIR);
-        let overlay = new Gtk.Overlay({ visible: true, can_focus: true });
+        let overlay = new Gtk.Overlay({ visible: true, can_focus: false });
         notificationManager = new NotificationManager.NotificationManager(overlay);
         this._mainWindow = new MainWindow.MainWindow(this, overlay);
         this._mainWindow.window.connect('destroy', this._onWindowDestroy.bind(this));
@@ -139,6 +170,7 @@ const Application = new Lang.Class({
 
     vfunc_activate: function() {
         this._createWindow();
+        this._checkNetwork();
         this._mainWindow.window.present();
     },
 
@@ -146,4 +178,3 @@ const Application = new Lang.Class({
         this._mainWindow = null;
     }
 });
-Utils.addSignalMethods(Application.prototype);
