@@ -20,18 +20,14 @@
  * Author: Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  */
 
-const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
-const Gio = imports.gi.Gio;
 const Geocode = imports.gi.GeocodeGlib;
-
+const Gio = imports.gi.Gio;
 const Lang = imports.lang;
-const Mainloop = imports.mainloop;
 
 const Application = imports.application;
 const Utils = imports.utils;
-const Path = imports.path;
-const Signals = imports.signals;
+
 const _ = imports.gettext.gettext;
 
 const ManagerInterface = '<node> \
@@ -80,6 +76,9 @@ const LocationProxy = Gio.DBusProxy.makeProxyWrapper(LocationInterface);
 const Geoclue = new Lang.Class({
     Name: 'Geoclue',
     Extends: GObject.Object,
+    Signals: {
+        'location-changed': { }
+    },
     Properties: {
         'connected': GObject.ParamSpec.boolean('connected',
                                                'Connected',
@@ -128,11 +127,15 @@ const Geoclue = new Lang.Class({
         let lastLocation = Application.settings.get('last-location');
         if (lastLocation.length >= 3) {
             let [lat, lng, accuracy] = lastLocation;
-            this.location = new Geocode.Location({ latitude: lat,
-                                                   longitude: lng,
-                                                   accuracy: accuracy });
             let lastLocationDescription = Application.settings.get('last-location-description');
-            this.location.set_description(lastLocationDescription);
+
+            let location = new Geocode.Location({ latitude: lat,
+                                                  longitude: lng,
+                                                  accuracy: accuracy });
+
+            this.place = new Geocode.Place({ location: location,
+                                             name: _("Current location") });
+
             this.userSetLocation = Application.settings.get('last-location-user-set');
         }
 
@@ -180,7 +183,7 @@ const Geoclue = new Lang.Class({
     },
 
     _updateLocation: function(location, userSet) {
-        this.location = location;
+        this.place.location = location;
 
         Application.settings.set('last-location', [location.latitude,
                                                    location.longitude,
@@ -194,4 +197,3 @@ const Geoclue = new Lang.Class({
         Utils.debug("Updated location: " + location.description);
     }
 });
-Utils.addSignalMethods(Geoclue.prototype);
