@@ -21,25 +21,23 @@
  *         Zeeshan Ali (Khattak) <zeeshanak@gnome.org>
  */
 
-const Gdk = imports.gi.Gdk;
 const GLib = imports.gi.GLib;
-const Gtk = imports.gi.Gtk;
-const Champlain = imports.gi.Champlain;
 const GObject = imports.gi.GObject;
-
+const Gdk = imports.gi.Gdk;
+const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const Mainloop = imports.mainloop;
 
 const Application = imports.application;
-const MapSource = imports.mapSource;
-const MapView = imports.mapView;
-const LayersPopover = imports.layersPopover;
+const Config = imports.config;
 const ContextMenu = imports.contextMenu;
+const FavoritesPopover = imports.favoritesPopover;
+const LayersPopover = imports.layersPopover;
+const MapView = imports.mapView;
 const PlaceEntry = imports.placeEntry;
 const PlaceStore = imports.placeStore;
 const Sidebar = imports.sidebar;
 const Utils = imports.utils;
-const Config = imports.config;
 const ZoomControl = imports.zoomControl;
 
 const _ = imports.gettext.gettext;
@@ -56,7 +54,12 @@ const MainWindow = new Lang.Class({
         let ui = Utils.getUIObject('main-window', [ 'app-window',
                                                     'header-bar',
                                                     'grid',
-                                                    'layers-button']);
+                                                    'main-stack',
+                                                    'no-network-view',
+                                                    'goto-user-location-button',
+                                                    'toggle-sidebar-button',
+                                                    'layers-button',
+                                                    'favorites-button' ]);
         this.window = ui.appWindow;
         this.window.application = app;
         this._overlay = overlay;
@@ -64,32 +67,30 @@ const MainWindow = new Lang.Class({
         this.mapView = new MapView.MapView();
         overlay.add(this.mapView);
 
-        overlay.add_overlay(new MapSource.AttributionLogo());
-
         this.mapView.gotoUserLocation(false);
 
-        this._sidebar = new Sidebar.Sidebar(this.mapView);
-        Application.routeService.route.connect('update',
-                                               this._setRevealSidebar.bind(this, true));
-        this._sidebar.bind_property('reveal-child',
-                                    this.mapView, 'routeVisible',
-                                    GObject.BindingFlags.DEFAULT);
+        this._sidebar = this._createSidebar();
 
         this._contextMenu = new ContextMenu.ContextMenu(this.mapView);
 
         ui.layersButton.popover = new LayersPopover.LayersPopover();
+        ui.favoritesButton.popover = new FavoritesPopover.FavoritesPopover({ mapView: this.mapView });
+        this._overlay.add_overlay(new ZoomControl.ZoomControl(this.mapView));
 
-        let placeEntry = this._createPlaceEntry();
-        ui.headerBar.set_custom_title(placeEntry);
-        placeEntry.has_focus = true;
+        this._mainStack = ui.mainStack;
+        this._mainStack.add(this._overlay);
+        this._noNetworkView = ui.noNetworkView;
+        this._headerBar = ui.headerBar;
+        this._gotoUserLocationButton = ui.gotoUserLocationButton;
+        this._toggleSidebarButton = ui.toggleSidebarButton;
+        this._layersButton = ui.layersButton;
+        this._favoritesButton = ui.favoritesButton;
 
+        this._initHeaderbar();
         this._initActions();
         this._initSignals();
         this._restoreWindowGeometry();
 
-        this._overlay.add_overlay(new ZoomControl.ZoomControl(this.mapView));
-
-        ui.grid.attach(this._overlay, 0, 0, 1, 1);
         ui.grid.attach(this._sidebar, 1, 0, 1, 1);
 
         ui.grid.show_all();
@@ -111,46 +112,63 @@ const MainWindow = new Lang.Class({
 
         let popover = placeEntry.popover;
         popover.connect('selected',
-                        this._overlay.grab_focus.bind(this._overlay));
+                        this.mapView.grab_focus.bind(this.mapView));
         this.mapView.view.connect('button-press-event',
                                   popover.hide.bind(popover));
         return placeEntry;
     },
 
+    _createSidebar: function() {
+        let sidebar = new Sidebar.Sidebar(this.mapView);
+        Application.routeService.query.connect('notify',
+                                               this._setRevealSidebar.bind(this, true));
+        sidebar.bind_property('reveal-child',
+                              this.mapView, 'routeVisible',
+                              GObject.BindingFlags.DEFAULT);
+        this.window.application.bind_property('connected',
+                                              sidebar, 'visible',
+                                              GObject.BindingFlags.DEFAULT);
+        return sidebar;
+    },
+
     _initActions: function() {
-        Utils.initActions(this.window, [
-            {
-                properties: { name: 'close' },
-                signalHandlers: { activate: this.window.close.bind(this.window) }
-            }, {
-                properties: { name: 'about' },
-                signalHandlers: { activate: this._onAboutActivate }
-            }, {
-                properties: {
-                    name: 'map-type-menu',
-                    state: GLib.Variant.new('b', false)
-                },
-                signalHandlers: { activate: this._onMapTypeMenuActivate }
-            }, {
-                properties: {
-                    name: 'map-type',
-                    parameter_type: GLib.VariantType.new('s'),
-                    state: GLib.Variant.new('s', 'STREET')
-                },
-                signalHandlers: { activate: this._onMapTypeActivate }
-            }, {
-                properties: { name: 'goto-user-location' },
-                signalHandlers: { activate: this._onGotoUserLocationActivate }
-            }, {
-                properties: {
-                    name: 'toggle-sidebar',
-                    state: GLib.Variant.new_boolean(false)
-                },
-                signalHandlers: {
-                    'change-state': this._onToggleSidebarChangeState
-                }
+        Utils.addActions(this.window, {
+            'close': {
+                onActivate: this.window.close.bind(this.window)
+            },
+            'about': {
+                onActivate: this._onAboutActivate.bind(this)
+            },
+            'map-type-menu': {
+                state: ['b', false],
+                onActivate: this._onMapTypeMenuActivate.bind(this)
+            },
+            'map-type': {
+                paramType: 's',
+                state: ['s', 'STREET'],
+                onActivate: this._onMapTypeActivate.bind(this)
+            },
+            'goto-user-location': {
+                onActivate: this._onGotoUserLocationActivate.bind(this)
+            },
+            'toggle-sidebar': {
+                accels: ['<Primary>D'],
+                state: ['b', false],
+                onChangeState: this._onToggleSidebarChangeState.bind(this)
+            },
+            'zoom-in': {
+                accels: ['<Primary>plus'],
+                onActivate: this.mapView.view.zoom_in.bind(this.mapView.view)
+            },
+            'zoom-out': {
+                accels: ['<Primary>minus'],
+                onActivate:  this.mapView.view.zoom_out.bind(this.mapView.view)
+            },
+            'find': {
+                accels: ['<Primary>F'],
+                onActivate: this._placeEntry.grab_focus.bind(this._placeEntry)
             }
-        ], this);
+        });
 
         let action = this.window.lookup_action('goto-user-location');
         Application.geoclue.bind_property('connected',
@@ -164,12 +182,38 @@ const MainWindow = new Lang.Class({
                             this._onConfigureEvent.bind(this));
         this.window.connect('window-state-event',
                             this._onWindowStateEvent.bind(this));
-        this.window.connect('key-press-event',
-                            this._onKeyPressEvent.bind(this));
-
         this.mapView.view.connect('button-press-event',
-                                  this._overlay.grab_focus.bind(this._overlay));
-        this._viewMovedId = 0;
+                                  this.mapView.grab_focus.bind(this.mapView));
+
+        this.window.application.connect('notify::connected', (function() {
+            if(this.window.application.connected)
+                this._mainStack.visible_child = this._overlay;
+            else
+                this._mainStack.visible_child = this._noNetworkView;
+        }).bind(this));
+    },
+
+    _initHeaderbar: function() {
+        this._placeEntry = this._createPlaceEntry();
+        this._headerBar.custom_title = this._placeEntry;
+        this._placeEntry.has_focus = true;
+
+        let favoritesPopover = this._favoritesButton.popover;
+        this._favoritesButton.sensitive = favoritesPopover.rows > 0;
+        favoritesPopover.connect('notify::rows', (function() {
+            this._favoritesButton.sensitive = favoritesPopover.rows > 0;
+        }).bind(this));
+
+        this.window.application.connect('notify::connected', (function() {
+            let app = this.window.application;
+
+            this._gotoUserLocationButton.sensitive = app.connected;
+            this._layersButton.sensitive = app.connected;
+            this._toggleSidebarButton.sensitive = app.connected;
+            this._favoritesButton.sensitive = (app.connected &&
+                                               favoritesPopover.rows > 0);
+            this._placeEntry.sensitive = app.connected;
+        }).bind(this));
     },
 
     _saveWindowGeometry: function() {
@@ -229,22 +273,6 @@ const MainWindow = new Lang.Class({
         Application.settings.set('window-maximized', maximized);
     },
 
-    _onKeyPressEvent: function(widget, event) {
-        let state = event.get_state()[1];
-
-        if (state & Gdk.ModifierType.CONTROL_MASK) {
-            let keyval = event.get_keyval()[1];
-
-            if (keyval === Gdk.KEY_plus)
-                this.mapView.view.zoom_in();
-
-            if (keyval === Gdk.KEY_minus)
-                this.mapView.view.zoom_out();
-        }
-
-        return false;
-    },
-
     _quit: function() {
         // remove configure event handler if still there
         if (this._configureId !== 0) {
