@@ -18,18 +18,16 @@
  * Author: Jonas Danielsson <jonas@threetimestwo.org>
  */
 
-const Geocode = imports.gi.GeocodeGlib;
 const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
-const Gio = imports.gi.Gio;
 const GdkPixbuf = imports.gi.GdkPixbuf;
+const Geocode = imports.gi.GeocodeGlib;
 const Gtk = imports.gi.Gtk;
-
 const Lang = imports.lang;
-const Mainloop = imports.mainloop;
 
-const Utils = imports.utils;
 const Application = imports.application;
+const Place = imports.place;
+const Utils = imports.utils;
 
 const _PLACES_STORE_FILE = 'maps-places.json';
 const _ICON_SIZE = 20;
@@ -86,22 +84,28 @@ const PlaceStore = new Lang.Class({
         this.set_sort_column_id(Columns.ADDED, Gtk.SortType.ASCENDING);
     },
 
-    addFavorite: function(place) {
-        if (this._exists(place, PlaceType.FAVORITE))
-            return;
-
-        if (this._exists(place, PlaceType.RECENT)) {
-            this._removeIf((function(model, iter) {
-                let p = model.get_value(iter, Columns.PLACE);
-                return p.name === place.name;
-            }), true);
-        }
-        this._addPlace(place, PlaceType.FAVORITE, new Date().getTime());
+    _addPlace: function(place, type) {
+        this._setPlace(this.append(), place, type, new Date().getTime());
+        this._store();
     },
 
-    addRecent: function(place) {
-        if (this._exists(place, PlaceType.RECENT)) {
-            this._updateAddTime(place);
+    _addFavorite: function(place) {
+        if (this.exists(place.osm_id, PlaceType.FAVORITE)) {
+            return;
+        }
+
+        if (this.exists(place.osm_id, PlaceType.RECENT)) {
+            this._removeIf((function(model, iter) {
+                let p = model.get_value(iter, Columns.PLACE);
+                return p.osm_id === place.osm_id;
+            }), true);
+        }
+        this._addPlace(place, PlaceType.FAVORITE);
+    },
+
+    _addRecent: function(place) {
+        if (this.exists(place.osm_id, PlaceType.RECENT)) {
+            this._updatePlace(place);
             return;
         }
 
@@ -112,15 +116,15 @@ const PlaceStore = new Lang.Class({
                 let type = model.get_value(iter, Columns.TYPE);
 
                 if (type === PlaceType.RECENT) {
-                    let name = model.get_value(iter, Columns.NAME);
-                    this._typeTable[name] = null;
+                    let place = model.get_value(iter, Columns.PLACE);
+                    this._typeTable[place.osm_id] = null;
                     this._numRecent--;
                     return true;
                 }
                 return false;
             }).bind(this), true);
         }
-        this._addPlace(place, PlaceType.RECENT, new Date().getTime());
+        this._addPlace(place, PlaceType.RECENT);
         this._numRecent++;
     },
 
@@ -134,27 +138,16 @@ const PlaceStore = new Lang.Class({
 
         try {
             let jsonArray = JSON.parse(buffer);
-            jsonArray.forEach((function(obj) {
-                let location = new Geocode.Location({
-                    latitude:    obj.latitude,
-                    longitude:   obj.longitude,
-                    altitude:    obj.altitude,
-                    accuracy:    obj.accuracy,
-                    description: obj.name
-                });
-                let place = Geocode.Place.new_with_location(obj.name,
-                                                            obj.place_type,
-                                                            location);
-                if (obj.bounding_box) {
-                    place.set_bounding_box(new Geocode.BoundingBox({
-                        top: obj.bounding_box.top,
-                        bottom: obj.bounding_box.bottom,
-                        left: obj.bounding_box.left,
-                        right: obj.bounding_box.right
-                    }));
-                }
-                this._addPlace(place, obj.type, obj.added);
-                if (obj.type === PlaceType.RECENT)
+            jsonArray.forEach((function({ place, type, added }) {
+                // We expect exception to be thrown in this line when parsing
+                // gnome-maps 3.14 or below place stores since the "place"
+                // key is not present.
+                if (!place.id)
+                    return;
+
+                let p = Place.Place.fromJSON(place);
+                this._setPlace(this.append(), p, type, added);
+                if (type === PlaceType.RECENT)
                     this._numRecent++;
             }).bind(this));
         } catch (e) {
@@ -162,34 +155,50 @@ const PlaceStore = new Lang.Class({
         }
     },
 
+    addPlace: function(place, type) {
+        if (type === PlaceType.FAVORITE)
+            this._addFavorite(place, type);
+        else if (type === PlaceType.RECENT)
+            this._addRecent(place, type);
+    },
+
+    removePlace: function(place, placeType) {
+        if (!this.exists(place.osm_id, placeType))
+            return;
+
+        this._removeIf((function(model, iter) {
+            let p = model.get_value(iter, Columns.PLACE);
+            if (p.osm_id === place.osm_id) {
+                this._typeTable[place.osm_id] = null;
+                return true;
+            }
+            return false;
+        }).bind(this), true);
+        this._store();
+    },
+
+    getModelForPlaceType: function(placeType) {
+        let filter = new Gtk.TreeModelFilter({ child_model: this });
+
+        filter.set_visible_func(function(model, iter) {
+            let type = model.get_value(iter, Columns.TYPE);
+            return (type === placeType);
+        });
+
+        return filter;
+    },
+
     _store: function() {
         let jsonArray = [];
         this.foreach(function(model, path, iter) {
-            let place    = model.get_value(iter, Columns.PLACE),
-                location = place.location,
-                type     = model.get_value(iter, Columns.TYPE),
-                added    = model.get_value(iter, Columns.ADDED);
-
-            let bounding_box = null;
-            if (place.bounding_box !== null) {
-                bounding_box = {
-                    top: place.bounding_box.top,
-                    bottom: place.bounding_box.bottom,
-                    left: place.bounding_box.left,
-                    right: place.bounding_box.right
-                };
-            }
+            let place = model.get_value(iter, Columns.PLACE);
+            let type = model.get_value(iter, Columns.TYPE);
+            let added = model.get_value(iter, Columns.ADDED);
 
             jsonArray.push({
-                place_type:   place.place_type,
-                name:         place.name,
-                latitude:     location.latitude,
-                longitude:    location.longitude,
-                altitude:     location.altitude,
-                accuracy:     location.accuracy,
-                bounding_box: bounding_box,
-                type:         type,
-                added:        added
+                place: place.toJSON(),
+                type: type,
+                added: added
             });
         });
 
@@ -198,9 +207,7 @@ const PlaceStore = new Lang.Class({
             log('Failed to write places file!');
     },
 
-    _addPlace: function(place, type, added) {
-        let iter = this.append();
-
+    _setPlace: function(iter, place, type, added) {
         this.set(iter,
                  [Columns.PLACE,
                   Columns.NAME,
@@ -216,12 +223,27 @@ const PlaceStore = new Lang.Class({
                 this.set(iter, [Columns.ICON], [pixbuf]);
             }).bind(this));
         }
-        this._typeTable[place.name] = type;
-        this._store();
+        this._typeTable[place.osm_id] = type;
     },
 
-    _exists: function(place, type) {
-        return this._typeTable[place.name] === type;
+    get: function(osmId) {
+        let place = null;
+        this.foreach(function(model, path, iter) {
+            let p = model.get_value(iter, Columns.PLACE);
+            if (p.osm_id === osmId) {
+                place = p;
+                return true;
+            }
+            return false;
+        });
+        return place;
+    },
+
+    exists: function(osmId, type) {
+        if (type !== undefined && type !== null)
+            return this._typeTable[osmId] === type;
+        else
+            return this._typeTable[osmId] !== undefined;
     },
 
     _removeIf: function(evalFunc, stop) {
@@ -235,13 +257,13 @@ const PlaceStore = new Lang.Class({
         }).bind(this));
     },
 
-    _updateAddTime: function(place) {
+    _updatePlace: function(place) {
         this.foreach((function(model, path, iter) {
-            let name = model.get_value(iter, Columns.NAME);
+            let p = model.get_value(iter, Columns.PLACE);
 
-            if (name === place.name) {
-                let updated = new Date().getTime();
-                model.set_value(iter, Columns.ADDED, updated);
+            if (p.osm_id === place.osm_id) {
+                let type = model.get_value(iter, Columns.TYPE);
+                this._setPlace(iter, place, type, new Date().getTime());
                 this._store();
                 return;
             }
