@@ -20,13 +20,10 @@
  * Author: Mattias Bengtsson <mattias.jc.bengtsson@gmail.com>
  */
 
-const Geocode = imports.gi.GeocodeGlib;
 const GObject = imports.gi.GObject;
-
+const Geocode = imports.gi.GeocodeGlib;
 const Lang = imports.lang;
 
-const Utils = imports.utils;
-
 const Transportation = {
     CAR:        0,
     BIKE:       1,
@@ -46,7 +43,9 @@ const RouteQuery = new Lang.Class({
     Name: 'RouteQuery',
     Extends: GObject.Object,
     Signals: {
-        'reset': { }
+        'reset': { },
+        'point-added': { param_types: [GObject.TYPE_OBJECT, GObject.TYPE_INT] },
+        'point-removed': { param_types: [GObject.TYPE_OBJECT, GObject.TYPE_INT] }
     },
     Properties: {
         'points': GObject.ParamSpec.object('points',
@@ -76,7 +75,7 @@ const RouteQuery = new Lang.Class({
 
     get filledPoints() {
         return this.points.filter(function(point) {
-            return point.place
+            return point.place;
         });
     },
 
@@ -86,16 +85,30 @@ const RouteQuery = new Lang.Class({
         this.reset();
     },
 
-    addPoint: function(point, index) {
+    addPoint: function(index) {
+        let point = new QueryPoint();
+
+        if (index === -1)
+            index = this.points.length - 1;
+
         this._points.splice(index, 0, point);
         point.connect('notify::place', (function() {
             this.notify('points');
         }).bind(this));
+        this.notify('points');
+        this.emit('point-added', point, index);
+
+        return point;
     },
 
     removePoint: function(index) {
-        this._points.splice(index, 1);
-        this.notify('points');
+        let removedPoints = this._points.splice(index, 1);
+        let point = removedPoints ? removedPoints[0] : null;
+
+        if (point) {
+            this.notify('points');
+            this.emit('point-removed', point, index);
+        }
     },
 
     set transportation(transportation) {
@@ -108,9 +121,11 @@ const RouteQuery = new Lang.Class({
 
     reset: function() {
         this.transportation = Transportation.CAR;
+        this.freeze_notify();
         this._points.forEach(function(point) {
             point.place = null;
         });
+        this.thaw_notify();
         this.emit('reset');
     },
 
@@ -139,6 +154,11 @@ const QueryPoint = new Lang.Class({
                                           Geocode.Place),
     },
 
+    _init: function() {
+        this._place = null;
+        this.parent();
+    },
+
     set place(p) {
         this._place = p;
         this.notify('place');
