@@ -21,21 +21,41 @@
  *         Mattias Bengtsson <mattias.jc.bengtsson@gmail.com>
  */
 
-const Geocode = imports.gi.GeocodeGlib;
-const GLib = imports.gi.GLib;
 const GObject = imports.gi.GObject;
 const Gtk = imports.gi.Gtk;
 const Lang = imports.lang;
 const _ = imports.gettext.gettext;
 
 const Application = imports.application;
-const PlaceStore = imports.placeStore;
 const PlaceEntry = imports.placeEntry;
-const Route = imports.route;
 const RouteQuery = imports.routeQuery;
-const SearchPopup = imports.searchPopup;
 const Utils = imports.utils;
 
+const InstructionRow = new Lang.Class({
+    Name: "InstructionRow",
+    Extends: Gtk.ListBoxRow,
+
+    _init: function(params) {
+        this.turnPoint = params.turnPoint;
+        delete params.turnPoint;
+
+        this.parent(params);
+
+        this.visible = true;
+        let ui = Utils.getUIObject('sidebar', ['instruction-box',
+                                               'direction-image',
+                                               'instruction-label',
+                                               'distance-label']);
+        ui.instructionLabel.label = this.turnPoint.instruction;
+        ui.directionImage.icon_name = this.turnPoint.iconName;
+
+        if (this.turnPoint.distance > 0)
+            ui.distanceLabel.label = Utils.prettyDistance(this.turnPoint.distance);
+
+        this.add(ui.instructionBox);
+    }
+});
+
 const Sidebar = new Lang.Class({
     Name: 'Sidebar',
     Extends: Gtk.Revealer,
@@ -79,15 +99,22 @@ const Sidebar = new Lang.Class({
                                         ui.modeBikeToggle,
                                         ui.modeCarToggle);
 
+        let query = Application.routeService.query;
+
+        query.addPoint(0);
         let fromEntry = this._initRouteEntry(ui.fromEntryGrid, 0);
+
+        query.addPoint(1);
         this._initRouteEntry(ui.toEntryGrid, 1);
 
+        this._initQuerySignals(ui.viaGridContainer);
+
         this.bind_property('child-revealed',
                            fromEntry, 'has_focus',
                            GObject.BindingFlags.DEFAULT);
 
         ui.viaAddButton.connect('clicked', (function() {
-            this._createViaRow(ui.viaGridContainer);
+            query.addPoint(-1);
         }).bind(this));
 
         this.add(ui.sidebar);
@@ -123,30 +150,44 @@ const Sidebar = new Lang.Class({
         query.connect('notify::transportation', setToggles);
     },
 
+    _initQuerySignals: function(listbox) {
+        let query = Application.routeService.query;
+
+        // Do nothing for the From and To points.
+        query.connect('point-added', (function(obj, point, index) {
+            if (index !== 0 && index !== query.points.length - 1)
+                this._createViaRow(listbox, index);
+        }).bind(this));
+
+        query.connect('point-removed', (function(obj, point, index) {
+            let row = listbox.get_row_at_index(index - 1);
+            row.destroy();
+        }).bind(this));
+    },
+
     _createPlaceEntry: function() {
         return new PlaceEntry.PlaceEntry({ visible: true,
                                            can_focus: true,
                                            hexpand: true,
                                            receives_default: true,
                                            mapView: this._mapView,
-                                           parseOnFocusOut: true });
+                                           parseOnFocusOut: true,
+                                           maxChars: 15 });
     },
 
-    _createViaRow: function(listbox) {
+    _createViaRow: function(listbox, index) {
         let ui = Utils.getUIObject('route-via-row', [ 'via-grid',
                                                       'via-remove-button',
                                                       'via-entry-grid' ]);
+        let insertIndex = index - 1;
+        let entry = this._createPlaceEntry();
 
-        // Always insert before 'To'
-        let insertIndex = Application.routeService.query.points.length - 1;
+        this._initRouteEntry(ui.viaEntryGrid, index);
         listbox.insert(ui.viaGrid, insertIndex);
-        this._initRouteEntry(ui.viaEntryGrid, insertIndex);
 
         ui.viaRemoveButton.connect('clicked', function() {
             let row = ui.viaGrid.get_parent();
             let pointIndex = row.get_index();
-
-            listbox.remove(row);
             Application.routeService.query.removePoint(pointIndex + 1);
         });
     },
@@ -155,11 +196,10 @@ const Sidebar = new Lang.Class({
         let entry = this._createPlaceEntry();
         container.add(entry);
 
-        let point = new RouteQuery.QueryPoint();
+        let point = Application.routeService.query.points[pointIndex];
         entry.bind_property('place',
                             point, 'place',
                             GObject.BindingFlags.BIDIRECTIONAL);
-        Application.routeService.query.addPoint(point, pointIndex);
 
         return entry;
     },
@@ -173,7 +213,6 @@ const Sidebar = new Lang.Class({
             this._instructionStack.visible_child = this._instructionWindow;
             this._viaGridContainer.get_children().forEach((function(row) {
                 query.removePoint(row.get_index() + 1);
-                row.destroy();
             }).bind(this));
         }).bind(this));
 
@@ -213,28 +252,3 @@ const Sidebar = new Lang.Class({
         this._distanceInfo.label = '';
     }
 });
-
-const InstructionRow = new Lang.Class({
-    Name: "InstructionRow",
-    Extends: Gtk.ListBoxRow,
-
-    _init: function(params) {
-        this.turnPoint = params.turnPoint;
-        delete params.turnPoint;
-
-        this.parent(params);
-
-        this.visible = true;
-        let ui = Utils.getUIObject('sidebar', ['instruction-box',
-                                               'direction-image',
-                                               'instruction-label',
-                                               'distance-label']);
-        ui.instructionLabel.label = this.turnPoint.instruction;
-        ui.directionImage.icon_name = this.turnPoint.iconName;
-
-        if (this.turnPoint.distance > 0)
-            ui.distanceLabel.label = Utils.prettyDistance(this.turnPoint.distance);
-
-        this.add(ui.instructionBox);
-    }
-});
