@@ -20,15 +20,13 @@
  * Author: Dario Di Nucci <linkin88mail@gmail.com>
  */
 
+const Clutter = imports.gi.Clutter;
 const Geocode = imports.gi.GeocodeGlib;
-const GObject = imports.gi.GObject;
-
 const Lang = imports.lang;
+const Mainloop = imports.mainloop;
 
 const Application = imports.application;
 const MapMarker = imports.mapMarker;
-const Path = imports.path;
-const Route = imports.route;
 const TurnPointBubble = imports.turnPointBubble;
 const Utils = imports.utils;
 
@@ -45,7 +43,7 @@ const TurnPointMarker = new Lang.Class({
     },
 
     _init: function(params) {
-        this._turnPoint = params.turnPoint
+        this._turnPoint = params.turnPoint;
         delete params.turnPoint;
 
         params.place = new Geocode.Place({
@@ -63,6 +61,27 @@ const TurnPointMarker = new Lang.Class({
                  y: Math.floor(this.height / 2) - 1 };
     },
 
+    goToAndSelect: function(animate) {
+        if (!animate) {
+            this.parent(animate);
+        } else {
+            let view = this._mapView.view;
+            let turnPointZoomLevel = 16;
+
+            view.goto_animation_mode = Clutter.AnimationMode.LINEAR;
+            view.goto_duration = 0;
+
+            Utils.once(view, 'animation-completed::go-to', (function() {
+                view.zoom_level = turnPointZoomLevel;
+                view.center_on(this.place.location.latitude,
+                               this.place.location.longitude);
+                this.selected = true;
+            }).bind(this));
+            view.go_to(this.place.location.latitude,
+                       this.place.location.longitude);
+        }
+    },
+
     _createBubble: function() {
         return new TurnPointBubble.TurnPointBubble({ turnPoint: this.turnPoint,
                                                      place: this.place,
